/**
 * Created by John on 11/17/2016.
 * Modified by TFG on 07/15/2020 .
 *
 * Common topology for the RabbitmQ exchanges and queues
 */
module.exports = function(rabbit, subscribeTo, config) {

    return rabbit.configure({

        // arguments used to establish a connection to a broker
        connection: config.rabbitConfig,

        // define the exchanges
        exchanges: [{
            name: "topic-fresenius-loader",
            type: "direct",
            autoDelete: true
        }],

        // setup the queues, only subscribing to the one this service
        // will consume messages from
        queues: [{              // Master items queue
            name: "topic-fresenius-loader-mtq",
            autoDelete: true,
            subscribe: true
        }, {                     // Clinic Profile queue
            name: "topic-fresenius-loader-cpq",
            autoDelete: true,
            subscribe: true
        }, {                     // Par Value queue
            name: "topic-fresenius-loader-pvq",
            autoDelete: true,
            subscribe: true
        }, {                     // Clinic Routing queue
            name: "topic-fresenius-loader-crq",
            autoDelete: true,
            subscribe: true
        }],

        // binds exchanges and queues to one another
        bindings: [{
            exchange: "topic-fresenius-loader",
            target: "topic-fresenius-loader-mtq",
            keys: ["loader.mt"]
        }, {
            exchange: "topic-fresenius-loader",
            target: "topic-fresenius-loader-cpq",
            keys: ["loader.cp"]
        }, {
            exchange: "topic-fresenius-loader",
            target: "topic-fresenius-loader-pvq",
            keys: ["loader.pv"]
        }, {
            exchange: "topic-fresenius-loader",
            target: "topic-fresenius-loader-crq",
            keys: ["loader.cr"]
        }]
    }).then(null, function(err) {
        console.error('Could not connect or configure:', err);
    });
};