/**
 * Created by will on 10/1/15.
 * Modified by TFG on 02/26/19 and 10/15/19.
 */
var fs = require('fs');

var awcblvalidation = require('./awcblvalidation');
var awcblcommon = require('./awcblcommon');

// Defines where the monitoring areas are
var validateFileDir = process.env.MASTER_FILE || '/opt/fresenius/efs/sftp/bcdsupport/dropbox/prepare';
var countFiles = process.env.COUNT_FILE || '/Users/will/dev/fresenius/fresenius-m3-api/components/data/counts/';

var validationFindFile = false;
var loadingFindFile = false;

var env = process.env.NODE_ENV || "dev";
var config = require('./config.json')[env];

//
// ------------------- Exporting functions --------------------------------------
//
module.exports.resetLoadingFlag = resetLoadingFlag;

//
// ------------------- Detection - entry point Bulk management -----------------------------
//

if (!String.prototype.endsWith) {
  String.prototype.endsWith = function(searchString, position) {
    var subjectString = this.toString();
    if (typeof position !== 'number' || !isFinite(position) || Math.floor(position) !== position || position > subjectString.length) {
      position = subjectString.length;
    }
    position -= searchString.length;
    var lastIndex = subjectString.indexOf(searchString, position);
    return lastIndex !== -1 && lastIndex === position;
  };
}

if (!String.prototype.startsWith) {
  String.prototype.startsWith = function(searchString, position) {
    position = position || 0;
    return this.indexOf(searchString, position) === position;
  };
}

module.exports = function(app) {
  var rh;
  var fileWithNoErrorsPromise;
  var RabbitHelper;
  var winston = app.get('winston');
  var logger = winston.loggers.get('main');
  var csv_line;

  RabbitHelper = require("../../components/helpers/RabbitHelper");

  fs.exists(validateFileDir, function(exists) {
    if (exists) {
      logger.info("Started watching validation area : " + validateFileDir);
      fs.watch(validateFileDir, function(event, filename) {
        if (filename !== null && validationFindFile === false) {
          // Set flag true to block further detection
          // Need to be set false after processing the file
          validationFindFile = true;

          if (filename.indexOf("MEDSExtract") > -1) {

            logger.info("Found Master Item bulk file (" + validateFileDir + "/" + filename + ") in the watching area ! Start validation");
            logger.info("Calling ValidateMasterItem");
            fileWithNoErrorsPromise = awcblvalidation.ValidateMasterItem(app, filename);
            fileWithNoErrorsPromise.then(response => {
              // Received response from validation process
              validationFindFile = false;
              if (response !== "FAIL") {
                rh = new RabbitHelper(winston, config);
                rh.processData(winston, response, "loader.mt");

                csv_line = response.toString();
                csv_line = csv_line.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
                logger.debug("TFG !!! ==> transaction ID generated  : " + JSON.stringify(csv_line[0], null, 4));

                awcblcommon.writeAwcblEntryLog(app,
                    filename,
                    csv_line[0].toString(),   // Transaction ID
                    "LOADING",
                    "Validation Passed. Loading the file.").then(response => {
                  logger.debug(response);
                }, reject => {
                  logger.error(reject);
                });
              } else {
                awcblcommon.writeAwcblEntryLog(app,
                    filename,
                    "N/A",
                    "FAILED",
                    "Validation Failed. Please view report.").then(response => {
                  logger.debug(response);
                }, reject => {
                  logger.error(reject);
                });
                logger.info("There are errors with the file. No message to publish. ");
              }
            }, err => {
              logger.info("Error with file processing. No message to publish.");
            });
          } else if (filename.indexOf("CHIPS_LOCATOR") > -1) {
            logger.info("Found Clinic Profile bulk file (" + validateFileDir + "/" + filename + ") in the watching area ! Start validation");
            logger.debug("Calling ValidateClinicProfile");
            fileWithNoErrorsPromise = awcblvalidation.ValidateClinicProfile(app, filename);

            fileWithNoErrorsPromise.then(response => {
              // Received response from validation process
              validationFindFile = false;

              if (response !== "FAIL") {
                rh = new RabbitHelper(winston, config);
                rh.processData(winston, response, "loader.cp");

                csv_line = response.toString();
                csv_line = csv_line.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
                logger.debug("TFG !!! ==> transaction ID generated  : " + JSON.stringify(csv_line[0], null, 4));

                awcblcommon.writeAwcblEntryLog(app,
                    filename,
                    csv_line[0].toString(),   // Transaction ID
                    "LOADING",
                    "Validation Passed. Loading the file.").then(response => {
                  logger.debug(response);
                }, reject => {
                  logger.error(reject);
                });
              } else {
                awcblcommon.writeAwcblEntryLog(app,
                    filename,
                    "N/A",
                    "FAILED",
                    "Validation Failed. Please view report.").then(response => {
                  logger.debug(response);
                }, reject => {
                  logger.error(reject);
                });
                logger.info("There are errors with the file. No message to publish. ");
              }
            }, err => {
              logger.info("Error with file processing. No message to publish.");
            });
          } else if (filename.indexOf("ITEMS_PAR_VALUE") > -1) {
            logger.debug("Found Item Par value bulk file (" + validateFileDir + "/" + filename + ") in the watching area ! Start validation");
            logger.debug("Calling ValidatePAR_VALUE");

            fileWithNoErrorsPromise = awcblvalidation.ValidateParValue(app, filename);
            fileWithNoErrorsPromise.then(response => {
              // Received response from validation process
              validationFindFile = false;
              if (response !== "FAIL") {
                rh = new RabbitHelper(winston, config);
                rh.processData(winston, response, "loader.pv");

                csv_line = response.toString();
                csv_line = csv_line.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
                logger.debug("TFG !!! ==> transaction ID generated  : " + JSON.stringify(csv_line[0],null, 4));

                awcblcommon.writeAwcblEntryLog(app, filename,
                    csv_line[0].toString(),
                    "LOADING",
                    "Validation Passed. Loading the file.").then(response => {
                  logger.debug(response);
                }, reject => {
                  logger.error(reject);
                });
              } else {
                awcblcommon.writeAwcblEntryLog(app, filename,
                    "N/A",
                    "FAILED",
                    "Validation Failed. Please view report.").then(response => {
                  logger.debug(response);
                }, reject => {
                  logger.error(reject);
                });
                logger.info("There are errors with the file. No message to publish. ");
              }
            }, err => {
              logger.info("Error with file processing. No message to publish.")
            });

          } else if (filename.indexOf("CLINIC_ROUTING") > -1) {
            logger.debug("Found Route Clinic bulk file (" + validateFileDir + "/" + filename + ") in the watching area ! Start validation");
            logger.debug("Calling ValidateClinicRouting");

            fileWithNoErrorsPromise = awcblvalidation.ValidateClinicRouting(app, filename);
            fileWithNoErrorsPromise.then(response => {
              // Received response from validation process
              validationFindFile = false;

              if (response !== "FAIL") {
                rh = new RabbitHelper(winston, config);
                rh.processData(winston, response, "loader.cr");

                csv_line = response.toString();
                csv_line = csv_line.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
                logger.debug("TFG !!! ==> transaction ID generated  : " + JSON.stringify(csv_line[0],null, 4));

                awcblcommon.writeAwcblEntryLog(app,
                    filename,
                    csv_line[0].toString(),
                    "LOADING",
                    "Validation Passed. Loading the file.").then(response => {
                  logger.debug(response);
                }, reject => {
                  logger.error(reject);
                });
              } else {
                logger.info("There are errors with the file. No message to publish. ");
                awcblcommon.writeAwcblEntryLog(app,
                    filename,
                    "N/A",
                    "FAILED",
                    "Validation Failed. Please view report.").then(response => {
                  logger.debug(response);
                }, reject => {
                  logger.error(reject);
                });
              }
            }, err => {
              logger.info("Error with file processing. No message to publish.");
            });

          } else {
            // NO VALID FILE FOUND !
            logger.debug("Loaded invalid file in the validation area !! - (" + validateFileDir + "/" + filename + ")");
            // Delete file since is not a valid one !  Notify the user ? How ?

            logger.debug("Remove Invalid File from Validation area");
            awcblvalidation.RemoveFileFromValidationArea(app, validateFileDir + "/" + filename);
            awcblcommon.writeAwcblEntryLog(app,
                "invalidFile.csv",
                "N/A",
                "FAILED",
                "Unrecognized file - erased from server").then(response => {
              logger.debug(response);
            }, reject => {
              logger.error(reject);
            });

            validationFindFile = false;
          }
        }
      });
    }
  });


  // Ancient code - never in use on Linux system
  // Keep for legacy and in case is needed in the future.  Never ported for Linux, never tested, no documentation
  // available about why was done and what exactly is supposed to do
  // No information available
  fs.exists(countFiles, function(exists) {
    if (exists) {
      logger.info("Started watching counting area: " + countFiles);

      fs.watch(countFiles, function(event, filename) {
        fs.readFile(countFiles + filename, 'utf8', function(err, data) {
          if (err) {
            logger.error("err: " + err);
          } else {
            awcblcommon.readCountData(app, data);
            fs.rename(countFiles + filename, countFiles + 'archive/' + filename, function(err) {
              if (err) {
                logger.error("err: " + err);
              }
            });
          }
        });
      });
    }
  });

  // Reset loading Flag
  // This function is exported to be used elsewhere in the code in order to reset the flag
  // for the loading file detection
  //    function resetLoadingFlag () {
  //        logger.info("Reset loading Find file flag");
  //        loadingFindFile = false;
  //    }
};

// Reset loading Flag
// This function is exported to be used elsewhere in the code in order to reset the flag
// for the loading file detection
function resetLoadingFlag(app) {
  var winston = app.get('winston');
  var logger = winston.loggers.get('main');

  logger.info("Reset loading Find file flag");
  loadingFindFile = false;
}