/**
 * Created by  TFG Jun, 3 2020
 */
// Controller for the upload

angular.module('app').controller('AwcblCtrl',  [
    '$scope',
    '$http',
    '$animate',
    'uiGridConstants',
    'Constants',
    'ngDialog',
    '$timeout',
    'FileSaver',
    'Blob',

    function($scope, $http, $animate, uiGridConstants, Constants, ngDialog, $timeout, FileSaver, Blob) {
        var requestData = {
            offset: 0,
            limit: 50,
            order: "id DESC"
        };
        var existingRow = {};

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.loadFile = {
            message: 'Loading selected file...',
            backdrop: true,
            busyPromise: null
        };

        $scope.busy = {
            message: 'Please wait...',
            backdrop: true,
            busyPromise: null
        };

        $scope.addActivityLog = {
            message: 'Add activity log...',
            backdrop: true,
            busyPromise: null
        };

        $scope.addAwcblLog = {
            message: 'Add bulk load entry log...',
            backdrop: true,
            busyPromise: null
        };

        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.useExternalSorting = true;
        $scope.gridOptions.useExternalPagination = true;
        $scope.gridOptions.useExternalFiltering = true;
        $scope.gridOptions.flatEntityAccess = true;
        $scope.gridOptions.columnDefs = Constants.awcblCols;
        $scope.gridOptions.onRegisterApi = function(gridApi) {
            $scope.gridApi = gridApi;
            $scope.gridApi.core.on.sortChanged($scope, function(grid, sortColumns) {
                onSortChanged(sortColumns);
            });
            $scope.gridApi.pagination.on.paginationChanged($scope, function(newPage, newSize) {
                requestData.offset = (newPage - 1) * newSize;
                requestData.limit = newSize;
                getPage();
            });
            $scope.gridApi.core.on.filterChanged($scope, onFilterChanged)
        };
        $scope.gridOptions.totalItems = 0;

        $scope.isDisabled = false;
        $scope.loading.busyPromise = getPage();

        $scope.digits = {};

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        function getPage() {
            console.log("Awcbl request data : " + JSON.stringify(requestData));
            $http.post('/api/getAwcblLogDataTotal', requestData)
              .then(function(res) {
                  $scope.gridOptions.totalItems = res.data;
                  console.log("AWCBL Log records count : " + res.data);

              }).catch(function(err) {
                console.log("ERROR ! - NO REFRESH on AWCBL count retrieval !!!!" + JSON.stringify(err));
            });

            return $http.post('/api/getAwcblLogData', requestData)
              .then(function(res){
                  $scope.gridOptions.data = null;
                  $scope.gridOptions.data = res.data;
                  console.log("Successfully retrieved AWCBL log data");
              }).catch(function(err) {
                  $scope.gridOptions.data = null;
                  console.log("ERROR ! - NO REFRESH on AWCBL data retrieval !!!!" + JSON.stringify(err));
              });
        }

        // ----------------- Loading file operations -----------------------------

        $scope.loadBulkFile = function () {
            console.log("Select file to upload");
            openLoadDialog();
        };

        // Open pop up dialog to load a file
        function openLoadDialog() {
            ngDialog.open({
                template: '/awcbl/awcblLoadFileView.html',
                showClose: true,
                className: "ngdialog-theme-default awcblPreview",
                scope: $scope
            });
        }

        // Pop up load file dialog callback
        $scope.add = function() {
            var file = document.getElementById('awcblfile').files[0];

            // Add activity log entry
            // leave here the code in case to add an activity entry log on loading a bulk file.
            //$scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_awcbl_log', 'loaded file : ' + JSON.stringify(file));

            // Add Bulk load log entry
            // Notify a user loaded a file
            $scope.addAwcblLog.busyPromise = addBulkLoadLogEntry(file.name,
                "--",
                "VALIDATION",
                "loaded in validation area on the server");

            $scope.files = [];
            $scope.files.push(file);

            console.log("Calling File Upload");

            return $http({
                method: 'POST',
                url: '/api/awcblUpload',
                headers: { 'Content-Type': undefined },
                transformRequest: function (data) {
                    var formData = new FormData();
                    formData.append('model', angular.toJson(data.model));
                    formData.append('file', data.files[0]);
                    $scope.closeDialog();
                    getPage();
                    return formData;
                },
                data: { model: { title: file.name }, files: $scope.files }

            }).success(function (res) {
                console.log(res);
                console.log("File Uploaded");
                $scope.closeDialog();
                getPage();
            });
        };

        $scope.GetReportLink = function (row) {
           if(row.entity.reportPath != null)
             return row.entity.reportPath;
        };


        // OLD MATERIAL - keep for reference for now

        $scope.addRowModal = function (row) {
            $scope.editRow = {};
            $scope.dataAction="Add";
            if (row !== null) {
                existingRow = findById($scope.gridOptions.data, row.entity.id);
                $scope.editRow = angular.copy(existingRow); //can't save changes to existing row
                $scope.dataAction="Update";
            }
            openEditDialog();
        };

        function openEditDialog() {
            //Using editVendor as a bindable Object in the modal.... cloning the original Vendor list
            //$scope.editVendor = angular.copy($scope.vendorList);
            $scope.editFieldsEnabled=true;
            ngDialog.open({
                template: '/awcbl/awcblView.html',
                showClose: true,
                className: "ngdialog-theme-default awcblPreview",
                scope: $scope
            });
        }

        $scope.exportData = function() {
            console.log("Export data : " + JSON.stringify(requestData));
            //$scope.busy.busyPromise = getActvityLogPromise();
        };

        $scope.editExistingRow = function() {
            $scope.editRow = existingRow;
            //$scope.isDisabled = true;
        };

        $scope.closeDialog = function(){
            $scope.editRow = existingRow;
            ngDialog.close();
            //$scope.isDisabled = false;
        }




        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        function onSortChanged(sortColumns) {
            if (sortColumns.length != 0) {
                requestData.order = "'" + sortColumns[0].name + "' " + sortColumns[0].sort.direction;
                getPage();
            }

        }

        function onFilterChanged() {
            var grid = $scope.gridApi.grid;

            for (var i = 0; i < grid.columns.length; i++) {
                var column = grid.columns[i];
                if (column.filters[0].term != null && column.filters[0].term != "") {
                    if (requestData["where"] == null) {
                        requestData["where"] = {};
                    }
                    requestData["where"][column.field] = column.filters[0].term;

                } else {
                    if (requestData["where"] != null && requestData["where"][column.field] != null) {
                        delete requestData["where"][column.field];
                    }
                }
            }
            getPage();
        }

        $scope.forceFilter = function() {
            return onFilterChanged();
        };

        function fromUpdated(date) {
            getPage();
        }

        function toUpdated(date) {
            getPage();
        }

        // FRES-128 - ActivityLog table management

        function getActvityLogPromise() {
            return $http.post('/api/v3/getActivityLogFile', requestData, {timeout: 60000}).then(function(res) {
                var file = new Blob([ res.data ], {
                    type : 'text/tab-separated-values;charset=utf-8a'
                });
                var m = moment().format("YYYYMMDD_hhmm");
                FileSaver.saveAs(file, 'Activity_log_' + m + '.csv');

                console.log("Successfully retrieved AWCBL data");
            }).catch(function(err) {
                console.log("ERROR ! - NO data extraction !!!!" + JSON.stringify(err));
            });
        }

        // Important ! Give at least a couple of seconds if you need to call this function back-to-back
        function addActivityLogEntry(name_table, desc)  {
            console.log("activityLog.js - addActivityLogEntry : " + name_table + " - " + desc);
            var record = {nameTable:"", description:"", dateModified:0};
            record.dateModified = new Date();
            record.nameTable = name_table;
            record.description = desc;

            return $http.post('/api/recordActivityLog', record).then(function (res) {
                console.log("log recorded");
            }, function (err) {
                console.log("error: " + err);
            });
        }

        // Adding entry to the bulk load log management
        // Store in the Bulk Load Log db information about the log operations
        function addBulkLoadLogEntry(path, transaction_id, status, notes)  {
            console.log("awcblLog.js - addBulkLoadLogEntry : " +
              path + " - " +
              transaction_id + " - " +
              status + " - " +
              notes + " - ");
            var record = {reportPath:"", transactionId:"", status:"", notes:"", date:0};
            record.date = new Date();
            record.reportPath = path;
            record.transactionId = transaction_id;
            record.status = status;
            record.notes = notes;

            return $http.post('/api/setAwcblLogData', record).then(function (res) {
                console.log("log recorded");
            }, function (err) {
                console.log("error: " + err);
            });
        }
    }
]);