/**
 * Created by will on 10/2/15.
 * Modified by TFG on 11/19/2019.
*/
angular.module('app').controller('ClinicProfileCtrl', [
    '$scope',
    '$rootScope',
    '$http',
    'uiGridConstants',
    'Constants',
    'ngDialog',
    'FileSaver',
    'Blob',

    function($scope, $rootScope, $http, uiGridConstants, Constants, ngDialog, FileSaver, Blob) {
        //these are needed for IE11
        if (!String.prototype.endsWith) {
            String.prototype.endsWith = function(searchString, position) {
                var subjectString = this.toString();
                if (typeof position !== 'number' || !isFinite(position) || Math.floor(position) !== position || position > subjectString.length) {
                    position = subjectString.length;
                }
                position -= searchString.length;
                var lastIndex = subjectString.indexOf(searchString, position);
                return lastIndex !== -1 && lastIndex === position;
            };
        }
        if (!String.prototype.startsWith) {
            String.prototype.startsWith = function(searchString, position) {
                position = position || 0;
                return this.indexOf(searchString, position) === position;
            };
        }

        var requestData = {};
        var existingRow = {};
        var recordEdited = false;   // FRES-128 - flag to discrimintate between adding and editing

        $scope.editFieldsEnabled = false;
        $scope.showVerifyAlert = false;

        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.columnDefs = Constants.clinicProfileCols;
        $scope.gridOptions.onRegisterApi = function( gridApi ) {
            $scope.gridApi = gridApi;
            $scope.gridApi.core.on.filterChanged($scope, onFilterChanged)
        };

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        $scope.ESAlist=[];
        $scope.OTHERlist=[];

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.addActivityLog = {
            message: 'Add activity log...',
            backdrop: true,
            busyPromise: null
        };

        $scope.vendorList = null;

        $scope.loading.busyPromise = getPage();

        $scope.digits = {};

        $scope.busy = {
            message: 'Saving Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.activity = 'Edit Data';

        $scope.addRowModal = function (row) {
            $scope.editRow = {};
            if (row != null) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
            }
            getVendors(row);

        };

        $scope.getLists = function() {
            var body = {};
            $scope.ESAlist=[];
            $scope.OTHERlist=[];
            body.location = $scope.editRow;
            $scope.loading.busyPromise = $http.post('/api/v2/getMasterItemAsDeviceCount', body).then(function (res) {
                if (res) {
                    var rawList = res.data;
                    if (rawList.length > 0) {
                        for( var r = 0; r < rawList.length; r++) {
                            var row = rawList[r];
                            if (row.count_type==='ESA') {
                                $scope.ESAlist.push(row);
                            }
                            else {
                                $scope.OTHERlist.push(row);
                            }
                        }
                    }
                }
                ngDialog.open({
                    template: '/clinicProfile/clinicProfileInventory.html',
                    showClose: true,
                    className: "ngdialog-theme-default clinicProfilePreview",
                    scope: $scope
                });
            });
        }

        $scope.countModal = function (row) {
            $scope.editRow = {};
            if (row != null) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
                $scope.getLists();
            }

        };

        $scope.deleteRow = function(row) {
            if (row != null) {
                if (row.entity) {
                    var key = { clinicId: row.entity.clinicId };
                    $scope.busy.busyPromise = $http.post('/api/deleteClinicProfile', key).then(function(res) {
                        $scope.busy.busyPromise = getPage();
                        // FRES-128 - add entry in the activity log
                        $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_clinic_profile', 'removed record [' + row.entity.clinicId + "]");
                    });
                }
            }
        };


        function getPage() {
            console.log("getting data");
            return $http.post('/api/v2/getClinicProfile', requestData).then(function (res) {
                $scope.gridOptions.data = res.data;
            }, function (err) {
                $scope.data = null;
            });
        }

        function openEditDialog(row) {
            //Using editVendor as a bindable Object in the modal.... cloning the original Vendor list
            $scope.editVendor = angular.copy($scope.vendorList);
            //Mark a new "Checked" field, based on vendorList string
            if (row != null) {
                //edit mode
                for (var v = 0; v < $scope.editVendor.length; v++) {
                    $scope.editVendor[v].checked = $scope.isCheckboxChecked($scope.editVendor[v].catalogId);
                }
            } else {
                //add mode
                for (var v = 0; v < $scope.editVendor.length; v++) {
                    $scope.editVendor[v].checked = $scope.editVendor[v].isDefault;
                }
            }
            ngDialog.open({
                template: '/clinicProfile/clinicProfileMaintenance.html',
                showClose: true,
                className: "ngdialog-theme-default masterItem",
                scope: $scope
            });
        }

        function getVendors(row) {
            if ($scope.vendorList == null) {
                console.log("getting Catalogs");
                return $http.post('/api/getCatalogList', null).then(function (res) {
                    $scope.vendorList = res.data;
                    openEditDialog(row);
                }, function (err) {
                    $scope.vendorList = null;
                });
            } else {
                openEditDialog(row);
            }
        }

        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        $scope.verifyClinicId = function() {
            console.log("verifying");
            recordEdited = false;       // Reset flag add/edit

            var row = {};
            row.clinicId = $scope.editRow.clinicId;
            $http.post('/api/getClinicProfile', row)
                .then(function(res) {
                    if (res.data[0] != null) {
                        existingRow = res.data[0];
                        $scope.showVerifyAlert = true;
                        recordEdited = true;
                    } else {
                        $scope.editRow.mirceraVisible = false;
                        $scope.editRow.mirceraIo = false;
                        $scope.editRow.areanespVisible = false;
                        $scope.editRow.areanespIo = false;
                        $scope.editRow.epogenVisible = false;
                        $scope.editRow.epogenIo = false;
                        $scope.editRow.retacritVisible = false;
                        $scope.editRow.retacritIo = false;
                        $scope.editRow.countInEach = false;
                        $scope.editFieldsEnabled = true;
                    }
                }, function(err) {
                    console.log("Error checking Clinic Id: " + err);
                    $scope.closeDialog();
                })
        };

        $scope.editExistingRow = function() {
            $scope.editRow = existingRow;
            $scope.showVerifyAlert = false;
            $scope.editFieldsEnabled = true;
//            setTimeout(function() {
//                    $scope.$apply()
//                },
//                1000
//            );
        };

        $scope.exportData = function() {
            var request = requestData;
            //delete request.limit;
            //delete request.offset;

            $scope.busy.busyPromise = getClinicProfilePromise();
        };

        function getClinicProfilePromise() {
            return $http.post('/api/v3/getClinicProfileFile/', requestData, {timeout: 60000}).then(function(res) {
                var file = new Blob([ res.data ], {
                    type: 'text/plain;charset=utf-8a'
                });
                var m = moment().format("YYYYMMDD_hhmm");
                FileSaver.saveAs(file, 'CHIPS_Locator_' + m + '.csv');
                //var b64 = res.data;
                //blobUtil.base64StringToBlob(b64).then(function (blob) {
                    // success
                //    var m = moment().format("YYYYMMDD_hhmm");
                //    FileSaver.saveAs(blob, 'CHIPS_LOCATOR_' + m + '.xlsx');
                //}).catch(function (err) {
                    // error
                 //   console.log(err);
                //});

            });
        };


        $scope.saveClinic = function(vendors) {
            $scope.editRow.dateModified = new Date();
            var clinic = $scope.editRow.clinicId;
            if (clinic.length == 4) {
                clinic = clinic + "BM";
            }
            if (clinic.indexOf(".") < 0) {
                clinic = "." + clinic;
            }
            $scope.editRow.clinicId = clinic;

            //Before saving, build "new" vendorList string
            $scope.editRow.vendorList = "";
            var firstEntry = true;
            for (var v = 0; v < vendors.length; v++ ) {
                if (vendors[v].checked) {
                    if (!firstEntry) {
                        $scope.editRow.vendorList += ",";
                    }
                    $scope.editRow.vendorList += vendors[v].id;
                    firstEntry = false;
                }
            }

            $scope.busy.busyPromise = $http.post('/api/v2/saveClinicProfile', $scope.editRow)
                .then(function(res) {
                    // FRES-128 - add entry in the activity log
                    var msg = recordEdited === false ? "added record [" : "modified record [";
                    $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_clinic_profile', msg + $scope.editRow.clinicId + "]");
                    recordEdited = false;       // Reset flag add/edit

                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();

                }, function(err) {
                    console.log("error: " + err);
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                    recordEdited = false;       // Reset flag add/edit
                });
        };

        $scope.closeDialog = function(){
            ngDialog.close();
            $scope.showVerifyAlert = false;
            $scope.editFieldsEnabled = false;
        };

        $scope.isCheckboxChecked = function (vendorName) {
            var rtn = false;
            if (vendorName == "-BLANK-") {
                rtn = ($scope.editRow.vendorList.startsWith(",") || $scope.editRow.vendorList.endsWith(",") || $scope.editRow.vendorList.indexOf(",,") > -1 ) ;
            }
            else {
                rtn = ( $scope.editRow.vendorList.trim() == vendorName );
                rtn = rtn || ($scope.editRow.vendorList.endsWith("," + vendorName));
                rtn = rtn || ($scope.editRow.vendorList.startsWith(vendorName + ","));
                rtn = rtn || ($scope.editRow.vendorList.indexOf("," + vendorName + ",") > -1);
            }
            return rtn;
        };

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function onFilterChanged() {
            var grid = $scope.gridApi.grid;

            for (var i = 0; i < grid.columns.length; i++) {
                var column = grid.columns[i];
                if (column.filters[0].term != null && column.filters[0].term != "") {
                    if (requestData["where"] == null) {
                        requestData["where"] = {};
                    }
                    requestData["where"][column.field] = column.filters[0].term;

                } else {
                    if (requestData["where"] != null && requestData["where"][column.field] != null) {
                        delete requestData["where"][column.field];
                    }
                }
            }
            getPage();
        }

        $scope.forceFilter = function() {
            return onFilterChanged();
        };

        // FRES-128 - add entry to the ActivityLog table
        // Important ! Give at least a couple of seconds if you need to call this function back-to-back
        function addActivityLogEntry(name_table, desc)  {
            console.log("clinicProfileCtrl.js - addActivityLogEntry : " + name_table + " - " + desc);
            var record = {nameTable:"", description:"", dateModified:0};
            record.dateModified = new Date();
            record.nameTable = name_table;
            record.description = desc;

            return $http.post('/api/recordActivityLog', record).then(function (res) {
                console.log("log recorded");
            }, function (err) {
                console.log("error: " + err);
            });
        }
    }
]);
