angular.module('app').controller('ReceiptCtrl', [
    '$scope',
    '$http',
    '$animate',
    'Constants',
    'FileSaver',
    'Blob',
    '$rootScope',
    '$window',
    'ngDialog',
    function($scope, $http, $animate, Constants, FileSaver, Blob, $rootScope, $window, ngDialog) {

        $scope.data = [];

        //initializing busy indicator for export
        $scope.busy = {
            message: 'Exporting Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        };

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        //view methods
        $scope.exportData = function() {
            var request = requestData;
            delete request.limit;
            delete request.offset;

            $scope.busy.busyPromise = getDataPromise();
        };

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        $scope.closeDialog = function(){
            ngDialog.close();
            $scope.showSapVerifyAlert = false;
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = false;
        };

        //initialization for grid and column headers
        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.useExternalSorting = true;
        $scope.gridOptions.useExternalPagination = true;
        $scope.gridOptions.useExternalFiltering = true;
        $scope.gridOptions.flatEntityAccess = true;
        $scope.gridOptions.columnDefs = Constants.receiptCols;

        $scope.gridOptions.onRegisterApi =  function( gridApi ) {
            $scope.gridApi = gridApi;
            $scope.gridApi.core.on.sortChanged($scope, function(grid, sortColumns) {
                onSortChanged(sortColumns);
            });
            $scope.gridApi.pagination.on.paginationChanged($scope, function(newPage, newSize) {
                requestData.offset = (newPage - 1) * newSize;
                requestData.limit = newSize;
                getPage();
            });
            $scope.gridApi.core.on.filterChanged($scope, onFilterChanged)
        };

        var requestData = {
            offset: 0,
            limit: 50,
            order: 'timestamp DESC'
        };

        $scope.loading.busyPromise=getPage();

        function getPage() {
           return $http.post('/api/getReceipt/', requestData)
                .then(function(res) {
                    console.log("getpagedone");
                    $scope.gridOptions.data = res.data;
                    $scope.gridOptions.totalItems = res.headers('x-total-count');
                });

        }

        function getDataPromise() {
            return $http.post('/api/v2/getReceiptFile/', requestData).then(function(res) {
                var file = new Blob([ res.data ], {
                    type : 'application/csv'
                });
                FileSaver.saveAs(file, 'receiptData.csv');
            });
        }

        function onSortChanged(sortColumns) {
            if (sortColumns.length != 0) {
                requestData.order = sortColumns[0].name + " " + sortColumns[0].sort.direction;
                getPage();
            }

        }

        function onFilterChanged() {
            var grid = $scope.gridApi.grid;

            for (var i = 0; i < grid.columns.length; i++) {
                var column = grid.columns[i];
                if (column.filters[0].term != null && column.filters[0].term != "") {
                    if (requestData["where"] == null) {
                        requestData["where"] = {};
                    }
                    requestData["where"][column.field] = column.filters[0].term;
                } else {
                    if (requestData["where"] != null && requestData["where"][column.field] != null) {
                        delete requestData["where"][column.field];
                    }
                }
            }
            $scope.loading.busyPromise = getPage();
        }

        $scope.addRowModal = function (row) {
            if (row) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
                openEditDialog();
            }
        };

        function openEditDialog() {
            if ($window.confirm('Are you sure you want to change this qty?')) {
                ngDialog.open({
                    template: '/receipt/receiptMaintenance.html',
                    showClose: true,
                    className: "ngdialog-theme-default receipt",
                    scope: $scope
                });
            }
        }

        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        $scope.saveReceipt = function() {
            $scope.editRow.modifiedDate = new Date();
            //Before saving, build "new" vendorList string
            $scope.editRow.modifiedBy = $rootScope.user.displayName;
            $scope.savingData.busyPromise = $http.post('/api/v2/saveReceiptQty', $scope.editRow)
                .then(function(res) {
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                }).catch(function(err) {
                    console.log("error: " + err);
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                });
        };

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.timestamp == null) {
                requestData.where.timestamp = {};
            }

            requestData.where.timestamp['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.timestamp == null) {
                requestData.where.timestamp = {};
            }

            requestData.where.timestamp['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");
            getPage();
        }
    }
]);