/**
 * Created by TFG on 9/19/2019.
 */
angular.module('app').controller('VendorParCtrl', [
    '$scope',
    '$window',
    '$http',
    'uiGridConstants',
    'Constants',
    'ngDialog',
    '$timeout',
    'FileSaver',
    'Blob',

    function($scope, $window, $http, uiGridConstants, Constants, ngDialog, $timeout, FileSaver, Blob) {
        var requestData = {};
        var requestExtraData = {};
        var existingRow = {};
        var recordEdited = false;   // FRES-128 - flag to discrimintate between adding and editing

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.savingData = {
            message: 'Saving Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.addActivityLog = {
            message: 'Add activity log...',
            backdrop: true,
            busyPromise: null
        };

        $scope.busy = {
            message: 'Please wait...',
            backdrop: true,
            busyPromise: null
        };

        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.columnDefs = Constants.vendorParCols;
        $scope.gridOptions.onRegisterApi = function(gridApi) {
            $scope.gridApi = gridApi;
        };
        $scope.gridOptions.totalItems = 0;

        function getPage() {
            return $http.post('/api/getVendorParList', requestExtraData).then(function(res){
                $scope.gridOptions.data = null;
                $scope["gridOptions"].data = res.data;
            },function(err) {
                $scope.gridOptions.data = null;
                console.log("NO REFRESH!!!!" + err);
            });
        }

        $scope.isDisabled = false;
        $scope.showVendorVerifyAlert = false;

        $scope.loading.busyPromise = getPage();

        $scope.digits = {};

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        function openEditDialog() {
            //Using editVendor as a bindable Object in the modal.... cloning the original Vendor list
            //$scope.editVendor = angular.copy($scope.vendorList);
            ngDialog.open({
                template: '/vendor/vendorParMaintenance.html',
                showClose: true,
                scope: $scope
            });
        }

        $scope.addRowModal = function (row) {
            $scope.editRow = { isDefault: false };
            if (row != null) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
            }
            openEditDialog();
        };

        $scope.deleteRow = function(row) {
//            console.log("deleting " + JSON.stringify(row.entity));
            console.log("deleting " + row.entity.vendorMaterial + " + " + row.entity.vendorId + " + " + row.entity.clinicId);

            if (row != null) {
                var doDelete = $window.confirm("Are you sure?");
                if (doDelete) {
                    $scope.loading.busyPromise = $http.post('/api/deleteParVendor', row.entity).then(function(res) {
                        // FRES-128 - add entry in the activity log
                        $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_sap_item_par_value', 'removed record [' +
                            row.entity.vendorMaterial + " + " + row.entity.vendorId + " + " + row.entity.clinicId + "]");

                        $scope.loading.busyPromise = getPage();
                    });
                }
            }
        };

        // Functionality to the button "Fill up table" - commented out in the html for now
        $scope.fillUpTable = function (row) {
            $scope.loading.busyPromise = $http.post('/api/v1/syncVendorParTable', null)
                .then(function(res) {
                    console.log("Called the sync vendor tables API function - OK");
                    $scope.loading.busyPromise = getPage();
                    ngDialog.open({
                        template: '/vendor/vendorConfirm.html',
                        showClose: true,
                        scope: $scope
                    });
                }, function(err) {
                    console.log("Error checking Vendor tables : " + err);
                    ngDialog.open({
                        template: '/vendor/vendorError.html',
                        showClose: true,
                        scope: $scope
                    });
                });
        };

        $scope.saveVendor = function(row) {
            console.log("saveVendor " + row.vendorMaterial + "+" + row.vendorId + "+" + row.clinicId);

            $scope.editRow.dateModified = new Date();
            $scope.savingData.busyPromise = $http.post('/api/saveParVendor', $scope.editRow)
                .then(function (res) {
                    // FRES-128 - add entry in the activity log
                    var msg = recordEdited === false ? "added record [" : "modified record [";
                    $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_sap_item_par_value', msg +
                        row.vendorMaterial + " + " + row.vendorId + " + " + row.clinicId + "]");

                    $scope.syncTables(row);     // Force sync tables

                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                    recordEdited = false;       // Reset flag add/edit

                }, function (err) {
                    console.log("error: " + err);
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                    recordEdited = false;       // Reset flag add/edit
                });
        };

        $scope.syncTables = function(row) {
            console.log("syncTables ");

            var delayInMilliseconds = 2500; //2.5 second

            // Delay the call to give time to the previous activity log to be stored

            setTimeout(function() {
                //your code to be executed after 1 second

                $scope.loading.busyPromise = $http.post('/api/v1/syncVendorParTable', null)
                    .then(function(res) {
                        console.log("Called the sync vendor tables API function - OK");
                        // FRES-128 - add entry in the activity log
                        $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_sap_item_par_value', 'filled up');

                    }, function(err) {
                        console.log("Error checking Vendor tables : " + err);
                        ngDialog.open({
                            template: '/vendor/vendorError.html',
                            showClose: true,
                            scope: $scope
                        });
                    });
            }, delayInMilliseconds);
        };

        $scope.verifyVendor = function() {
            var row = {};
            row.vendorMaterial = $scope.editRow.vendorMaterial;
            row.vendorId = $scope.editRow.vendorId;
            row.clinicId = $scope.editRow.clinicId;

            console.log("verifying " + row.vendorMaterial + "+" + row.vendorId + "+" + row.clinicId);

            //query for the combo
            $http.post('/api/getParVendor', row)
                .then(function(res) {
                    if (res.data[0] != null) {
                        existingRow = res.data[0];

                        if ((existingRow.vendorMaterial != null && existingRow.vendorMaterial == row.vendorMaterial) &&
                            (existingRow.vendorId != null && existingRow.vendorId == row.vendorId) &&
                            (existingRow.clinicId != null && existingRow.clinicId == row.clinicId)){
                            $scope.showVendorVerifyAlert = true;
                            recordEdited = true;       // Set flag add/edit
                        }
                    } else {
                        $scope.isDisabled = true;
                    }
                }, function(err) {
                    console.log("Error checking Vendor Par Value : " + err);
                    $scope.closeDialog();
                });
        };

        $scope.editExistingRow = function() {
            console.log("editing");
            $scope.editRow = existingRow;
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = true;
        };

        $scope.exportData = function() {
            var request = requestData;

            $scope.busy.busyPromise = getItemParValuePromise();
        };

        function getItemParValuePromise() {
            return $http.post('/api/v3/getItemsParValueFile/', requestData, {timeout: 60000}).then(function(res) {
                var file = new Blob([ res.data ], {
                    type : 'text/tab-separated-values;charset=utf-8a'
                });
                var m = moment().format("YYYYMMDD_hhmm");
                FileSaver.saveAs(file, 'ITEMS_PAR_VALUE_' + m + '.csv');
                //var b64 = res.data;
                //blobUtil.base64StringToBlob(b64).then(function (blob) {
                // success
                //    var m = moment().format("YYYYMMDD_hhmm");
                //    FileSaver.saveAs(blob, 'M3Extract_' + m + '.ods');
                //}).catch(function (err) {
                // error
                //    console.log(err);
                //});
            });
        }

        $scope.closeDialog = function(){
            ngDialog.close();
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = false;
        };

        // FRES-128 - add entry to the ActivityLog table
        // Important ! Give at least a couple of seconds if you need to call this function back-to-back
        function addActivityLogEntry(name_table, desc)  {
            console.log("vendorParCtrl.js - addActivityLogEntry : " + name_table + " - " + desc);
            var record = {nameTable:"", description:"", dateModified:0};
            record.dateModified = new Date();
            record.nameTable = name_table;
            record.description = desc;

            return $http.post('/api/recordActivityLog', record).then(function (res) {
                console.log("log recorded");
                }, function (err) {
                   console.log("error: " + err);
                });
        }

        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");
            $scope.loading.busyPromise = getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");
            $scope.loading.busyPromise = getPage();
        }
    }
]);