/**
 * Created by TFG on 2/4/2020.
 * Modified by TFG on 5/21/2020 .
 */

angular.module('app').controller('VendorRouteClinicCtrl', [
    '$scope',
    '$window',
    '$http',
    'uiGridConstants',
    'Constants',
    'ngDialog',
    '$timeout',
    'FileSaver',
    'Blob',

    function($scope, $window, $http, uiGridConstants, Constants, ngDialog, $timeout, FileSaver, Blob) {
        var requestData = {};
        var requestExtraData = {};
        var existingRow = {};

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        }

        $scope.savingData = {
            message: 'Saving Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.busy = {
            message: 'Please wait...',
            backdrop: true,
            busyPromise: null
        };

        $scope.addActivityLog = {
            message: 'Add activity log...',
            backdrop: true,
            busyPromise: null
        };

        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.columnDefs = Constants.vendorRouteClinicCols;
        $scope.gridOptions.onRegisterApi = function(gridApi) {
            $scope.gridApi = gridApi;
        };
        $scope.gridOptions.totalItems = 0;

        function getPage() {
            return $http.post('/api/getVendorRouteClinicList', requestExtraData).then(function(res){
                $scope.gridOptions.data = null;
                $scope["gridOptions"].data = res.data;
            },function(err) {
                $scope.gridOptions.data = null;
                console.log("NO REFRESH!!!!" + err);
            });
        }

        $scope.isDisabled = false;
        $scope.showVendorVerifyAlert = false;

        $scope.loading.busyPromise = getPage();

        $scope.digits = {};

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        function openEditDialog() {
            //Using editVendor as a bindable Object in the modal.... cloning the original Vendor list
            //$scope.editVendor = angular.copy($scope.vendorList);
            ngDialog.open({
                template: '/vendor/vendorRouteClinicMaintenance.html',
                showClose: true,
                scope: $scope
            });
        }

        $scope.exportData = function() {
            var request = requestData;

            $scope.busy.busyPromise = getRouteClinicPromise();
        };

        function getRouteClinicPromise() {
            return $http.post('/api/v3/getVendorRouteClinicFile', requestData, {timeout: 60000}).then(function(res) {
                var file = new Blob([ res.data ], {
                    type : 'text/tab-separated-values;charset=utf-8a'
                });
                var m = moment().format("YYYYMMDD_hhmm");
                FileSaver.saveAs(file, 'CLINIC_ROUTING_' + m + '.csv');
                //var b64 = res.data;
                //blobUtil.base64StringToBlob(b64).then(function (blob) {
                // success
                //    var m = moment().format("YYYYMMDD_hhmm");
                //    FileSaver.saveAs(blob, 'M3Extract_' + m + '.ods');
                //}).catch(function (err) {
                // error
                //    console.log(err);
                //});
            });
        }


        $scope.addRowModal = function (row) {
            $scope.editRow = { isDefault: false };
            if (row != null) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
            }
            openEditDialog();
        };

        $scope.deleteRow = function(row) {
            if (row != null) {
                var doDelete = $window.confirm("Are you sure?");
                if (doDelete) {
                    $scope.loading.busyPromise = $http.post('/api/deleteRouteClinicVendor', row.entity).then(function (res) {
                        // FRES-128 - add entry in the activity log
                        $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_vendor_route_clinic', 'Deleted record [Clinic : ' +
                            $scope.editRow.clinicId + " - Vendor : " + $scope.editRow.vendorId + " - Route : " + $scope.editRow.vendorPathRoute + "]");

                        // syncAndRebuidVendorRouteProductTable();    // FRES-149 - regenerate routing product table
                        syncAndRebuidVendorRouteAppTable();           // FRES-169 - regenerate routing app table

                        $scope.loading.busyPromise = getPage();
                    }, function(err) {
                        console.log("error: " + err);
                        $scope.closeDialog();
                        $scope.loading.busyPromise = getPage();
                    });
                }
            }
        };

        $scope.saveVendor = function(row) {
            $scope.editRow.dateModified = new Date();
            $scope.savingData.busyPromise = $http.post('/api/saveRouteClinicVendor', $scope.editRow)
                .then(function(res) {
                    $scope.closeDialog();

                    // FRES-128 - add entry in the activity log
                    $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_vendor_route_clinic', 'Modified/Added record [Clinic : ' +
                        $scope.editRow.clinicId + " - Vendor : " + $scope.editRow.vendorId + " - Route : " + $scope.editRow.vendorPathRoute + "]");

                    // syncAndRebuidVendorRouteProductTable();    // FRES-149 - regenerate routing product table
                    syncAndRebuidVendorRouteAppTable();           // FRES-169 - regenerate routing app table

                    $scope.loading.busyPromise = getPage();
                }, function(err) {
                    console.log("error: " + err);
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                });
        };

        $scope.verifyVendor = function() {
            console.log("verifying");
            var row = {};
            row.clinicId = $scope.editRow.clinicId;
            row.vendorId = $scope.editRow.vendorId;

            //query for the combo
            $http.post('/api/getRouteClinicVendor', row)
                .then(function(res) {
                    if (res.data[0] != null) {
                        existingRow = res.data[0];
                        if (existingRow.vendorId != null && existingRow.vendorId == row.vendorId) {
                            $scope.showVendorVerifyAlert = true;
                        }
                    } else {
                        $scope.isDisabled = true;
                    }
                }, function(err) {
                    console.log("Error checking Vendor Route : " + err);
                    $scope.closeDialog();
                });
        };

        $scope.editExistingRow = function() {
            console.log("editing");
            $scope.editRow = existingRow;
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = true;
        };

        $scope.closeDialog = function(){
            ngDialog.close();
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = false;
            $scope.loading.busyPromise = getPage();
        };

        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");
            getPage();
        }

        // FRES-112/FRES-149
        // Function to sync tbl_vendor_list with tbl_vendor_route and then rebuild the tbl_vendor_route_product, the
        // table sent to the terminal.
        // The function is called everytime a new record is added to the tbl_vendor_list or when a record is
        // removed from the tbl_vendor_list
        // NOTE ! The function uses a dialog from the vendor page to notify errors or the sync completion
        function syncAndRebuidVendorRouteProductTable() {
            var delayInMilliseconds = 2500; //2.5 second
            // Delay the call to give time to the previous activity log to be stored

            console.log("syncAndRebuildVendorRouteProductTable -> starting");

            setTimeout(function() {
                $http.post('/api/v1/syncVendorRouteTables', null)
                    .then(function(res) {
                        console.log("Called the sync vendor route tables API function - OK");

                        $http.post('/api/v1/rebuildVendorRouteProductTable', null)
                            .then(function(res) {
                                console.log("Called the rebuild vendor route product tables API function - OK");

                                // Left the log entry writing commented out here but the log entry is actually done
                                // in the api.js in order to report the correct result of the rebuilding.
                                // FRES-128 - add entry in the activity log
//                                $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_vendor_route_product', 're-done the sync');

                                ngDialog.open({
                                    template: '/vendor/vendorConfirm.html',
                                    showClose: true,
                                    scope: $scope
                                });
                            }, function(err) {
                                console.log("Error rebuild Vendor tables : " + err);
                                ngDialog.open({
                                    template: '/vendor/vendorError.html',
                                    showClose: true,
                                    scope: $scope
                                });
                            });
                    }, function(err) {
                        console.log("Error synching Vendor tables : " + err);

                        ngDialog.open({
                            template: '/vendor/vendorError.html',
                            showClose: true,
                            scope: $scope
                        });
                        console.log("Error checking Vendor tables : " + err);
                    });
            }, delayInMilliseconds);
        }

        // FRES-169
        // Function to sync tbl_vendor_list with tbl_vendor_route and then rebuild the tbl_vendor_route_app, the
        // table sent to the terminal.
        // The function is called everytime a new record is added to the tbl_vendor_list or when a record is
        // removed from the tbl_vendor_list
        // NOTE ! The function uses a dialog from the vendor page to notify errors or the sync completion
        function syncAndRebuidVendorRouteAppTable() {
            // Delay the call to give time to the previous activity log to be stored
            var delayInMilliseconds = 2500; //2.5 second

            console.log("syncAndRebuildVendorRouteAppTable -> starting");

            setTimeout(function() {
                $http.post('/api/v1/syncVendorRouteTables', null)
                    .then(function(res) {
                        console.log("--> OK <-- Syncronized the main routing tables");

                        // Set flag to rebuild the routing table
                        $http.post('/api/setFlagNeedRoutingRebuilding', null)
                            .then(function(res) {
                                ngDialog.open({
                                    template: '/vendor/vendorConfirm.html',
                                    showClose: true,
                                    scope: $scope
                                });
                            }, function(err) {
                                console.log("Error setting flag (???) : " + err);
                                ngDialog.open({
                                    template: '/vendor/vendorError.html',
                                    showClose: true,
                                    scope: $scope
                                });
                            });
                    }, function(err) {
                        console.log("Error syncronizing main routing tables : " + err);
                        ngDialog.open({
                            template: '/vendor/vendorError.html',
                            showClose: true,
                            scope: $scope
                        });
                    });
            }, delayInMilliseconds);
        }
        
        // FRES-128 - add entry to the ActivityLog table
        // Important ! Give at least a couple of seconds if you need to call this function back-to-back
        function addActivityLogEntry(name_table, desc)  {
            var record = {nameTable:"", description:"", dateModified:0};
            record.dateModified = new Date();
            record.nameTable = name_table;
            record.description = desc;

            console.log("vendorRouteClinicCtrl.js - addActivityLogEntry : " + JSON.stringify(record, null, 4));

            return $http.post('/api/recordActivityLog', record).then(function (res) {
                console.log("log recorded");
            }, function (err) {
                console.log("error: " + err);
            });
        }
    }

]);