/**
 * Created by John on 4/4/2017.
 */
var cluster = require('cluster');
var clusterManager = require('cluster-manager');
var port = process.env.PORT || 3000;
var numCPUs = require('os').cpus().length;
var env = process.env.NODE_ENV || "dev";
var path = require('path');

var config      = require("./components/config/config.json")[env];
//initialize logging
var winston = require('winston');
var win = require('./components/config/logging')(winston, config);
app.set('winston', win);
var logger = win.loggers.get("main");

var childpath = path.join(__dirname, 'child.js');

if (cluster.isMaster) {
    var forks = (env === "dev") ? 2 : numCPUs;

    logger.debug("/n-------------------------------------------------------------------");
    logger.debug('Environment : ' + env + " Forks : " + forks + " Num cpus : " + numCPUs);

    cluster.on('online', function(worker) {
        console.log('Worker ' + worker.process.pid + ' is online');
    });
    cluster.on('exit', function(worker, code, signal) {
        if (worker.exitedAfterDisconnect === true) {
            logger.debug('Oh, it was just voluntary – no need to worry');
        }
        if (signal) {
            logger.debug('worker was killed by signal: ' + signal);
        } else if (code !== 0) {
            logger.debug('worker exited with error code: ' + code);
        } else {
            logger.debug('worker success!');
        }
    });

    clusterManager({
        exec: childpath,
        workers: forks,
        notify: null
    });

}

process.on('uncaughtException', function (err) {
    var msg = (new Date).toUTCString() + ' uncaughtException:';
    console.error(msg, err.message);
    console.error(err.stack);
    process.exit(1);
});